#
# Copyright 2005 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# Test the NamespaceMixin's set_value method
#
# $Id: set_value.py 15353 2010-01-18 09:09:09Z andrewm $

import unittest
import albatross

set_value = albatross.NamespaceMixin.set_value.im_func

class NS:
    pass

class SetValueTest(unittest.TestCase):

    def setUp(self):
        self.locals = NS()

    def _test(self, name, value):
        set_value(self, name, value)
        evaled = eval(name, {}, self.locals.__dict__)
        self.assertEqual(evaled, value)
    
    def test_set(self):
        self._test('a', 1)

    def test_errors(self):
        self.locals.b = 1
        self.assertRaises(AttributeError, set_value, self, 'a.b', 1)
        self.assertRaises(ValueError, set_value, self, 'b[a]', 1)
        self.assertRaises(SyntaxError, set_value, self, 'b[.a]', 1)
        self.assertRaises(SyntaxError, set_value, self, 'b[1.0]', 1)
        self.assertRaises(SyntaxError, set_value, self, 'b[1', 1)
        self.assertRaises(SyntaxError, set_value, self, 'b]', 1)
        self.assertRaises(SyntaxError, set_value, self, '.b', 1)
        self.assertRaises(SyntaxError, set_value, self, 'b.', 1)
        self.assertRaises(SyntaxError, set_value, self, '[0]', 1)

    def test_set_array(self):
        self.locals.b = [0] * 4
        self._test('b[0]', 1)
        self._test('b[2]', 2)
        self._test('b[-1]', 3)
        self.assertRaises(IndexError, set_value, self, 'b[4]', 1)
        self.assertEqual(self.locals.b, [1, 0, 2, 3])

    def test_set_sub(self):
        self.locals.c = NS()
        self.locals.c.e = [0] * 4
        f = NS()
        self.locals.c.e[3] = f
        self._test('c.d', 1)
        self._test('c.e[0]', 1)
        self._test('c.e[2]', 2)
        self._test('c.e[3].g', 3)
        self.assertRaises(IndexError, set_value, self, 'c.e[4]', 1)
        self.assertEqual(self.locals.c.e, [1, 0, 2, f])

    def test_iter_backdoor(self):
        class Iter:
            def set_backdoor(self, op, value):
                self.op = op
                self.value = value
        self.locals.f = Iter()
        self.locals.g = None
        set_value(self, 'x,f', 1)
        self.assertEqual(self.locals.f.op, 'x')
        self.assertEqual(self.locals.f.value, 1)
        self.assertRaises(AttributeError, set_value, self, 'x,g', 1)

    def test_array_array(self):
        self.locals.g = [[0] * 3] * 3
        self._test('g[1][2]', 2)


if __name__ == '__main__':
    unittest.main()

