#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# SessionServer functionality tests
#
# $Id: server.py 15367 2010-01-20 08:10:41Z andrewm $
#

import os, sys

import unittest
import albatross

import session_common

class Vars:
    pass

class SessionServerApp(albatross.SessionServerAppMixin):

    def module_path(self):
        pass

class SessionServer(albatross.ResponseMixin, albatross.SessionServerContextMixin):
    """ 
    This corresponds to the object albatross users know as the ctx,
    although since we aren't using the normal set of mixins, we have to
    supply our own locals and request members.
    """
    def __init__(self, request):
        self.app = SessionServerApp('test')
        self.request = request
        self.locals = Vars()
        albatross.ResponseMixin.__init__(self)
        albatross.SessionServerContextMixin.__init__(self)

    def parsed_request_uri(self):
        return '', '', ''

    def absolute_base_url(self):
        return ''

    def test_save(self):
        self.write_headers()
        self.save_session()


class Session:
    """
    This represents a virtual "user" - essentially just a place to store
    their session id while the ctx objects come and go.
    """
    def __init__(self):
        self.request = session_common.DummyCookieRequest()

    def new_context(self):
        return SessionServer(self.request)


# We are unable to start a session server in a portable manner, so if no
# session server is running on the standard port, we skip these tests.

def have_server():
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        try:
            s.connect(('127.0.0.1', 34343))
        except socket.error:
            return False
        else:
            return True
    finally:
        s.close()

if have_server():
    class ServerCase(session_common.SessionCommon):

        def setUp(self):
            pass

        def tearDown(self):
            pass

        def new_session(self):
            return Session()

else:
    print 'SKIPPED sessions.server.ServerCase: no session server'

if __name__ == '__main__':
    unittest.main()
